// SPDX-License-Identifier: GPL-2.0
/*
 *  linux/fs/ldiskfs/symlink.c
 *
 * Only fast symlinks left here - the rest is done by generic code. AV, 1999
 *
 * Copyright (C) 1992, 1993, 1994, 1995
 * Remy Card (card@masi.ibp.fr)
 * Laboratoire MASI - Institut Blaise Pascal
 * Universite Pierre et Marie Curie (Paris VI)
 *
 *  from
 *
 *  linux/fs/minix/symlink.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 *
 *  ldiskfs symlink handling code
 */

#include <linux/fs.h>
#include <linux/namei.h>
#include "ldiskfs.h"
#include "xattr.h"

static const char *ldiskfs_encrypted_get_link(struct dentry *dentry,
					   struct inode *inode,
					   struct delayed_call *done)
{
	struct buffer_head *bh = NULL;
	const void *caddr;
	unsigned int max_size;
	const char *paddr;

	if (!dentry)
		return ERR_PTR(-ECHILD);

	if (ldiskfs_inode_is_fast_symlink(inode)) {
		caddr = LDISKFS_I(inode)->i_data;
		max_size = sizeof(LDISKFS_I(inode)->i_data);
	} else {
		bh = ldiskfs_bread(NULL, inode, 0, 0);
		if (IS_ERR(bh))
			return ERR_CAST(bh);
		if (!bh) {
			LDISKFS_ERROR_INODE(inode, "bad symlink.");
			return ERR_PTR(-EFSCORRUPTED);
		}
		caddr = bh->b_data;
		max_size = inode->i_sb->s_blocksize;
	}

	paddr = fscrypt_get_symlink(inode, caddr, max_size, done);
	brelse(bh);
	return paddr;
}

static int ldiskfs_encrypted_symlink_getattr(struct user_namespace *mnt_userns,
					  const struct path *path,
					  struct kstat *stat, u32 request_mask,
					  unsigned int query_flags)
{
	ldiskfs_getattr(mnt_userns, path, stat, request_mask, query_flags);

	return fscrypt_symlink_getattr(path, stat);
}

static void ldiskfs_free_link(void *bh)
{
	brelse(bh);
}

static const char *ldiskfs_get_link(struct dentry *dentry, struct inode *inode,
				 struct delayed_call *callback)
{
	struct buffer_head *bh;
	char *inline_link;

	/*
	 * Create a new inlined symlink is not supported, just provide a
	 * method to read the leftovers.
	 */
	if (ldiskfs_has_inline_data(inode)) {
		if (!dentry)
			return ERR_PTR(-ECHILD);

		inline_link = ldiskfs_read_inline_link(inode);
		if (!IS_ERR(inline_link))
			set_delayed_call(callback, kfree_link, inline_link);
		return inline_link;
	}

	if (!dentry) {
		bh = ldiskfs_getblk(NULL, inode, 0, LDISKFS_GET_BLOCKS_CACHED_NOWAIT);
		if (IS_ERR(bh))
			return ERR_CAST(bh);
		if (!bh || !ldiskfs_buffer_uptodate(bh))
			return ERR_PTR(-ECHILD);
	} else {
		bh = ldiskfs_bread(NULL, inode, 0, 0);
		if (IS_ERR(bh))
			return ERR_CAST(bh);
		if (!bh) {
			LDISKFS_ERROR_INODE(inode, "bad symlink.");
			return ERR_PTR(-EFSCORRUPTED);
		}
	}

	set_delayed_call(callback, ldiskfs_free_link, bh);
	nd_terminate_link(bh->b_data, inode->i_size,
			  inode->i_sb->s_blocksize - 1);
	return bh->b_data;
}

const struct inode_operations ldiskfs_encrypted_symlink_inode_operations = {
	.get_link	= ldiskfs_encrypted_get_link,
	.setattr	= ldiskfs_setattr,
	.getattr	= ldiskfs_encrypted_symlink_getattr,
	.listxattr	= ldiskfs_listxattr,
};

const struct inode_operations ldiskfs_symlink_inode_operations = {
	.get_link	= ldiskfs_get_link,
	.setattr	= ldiskfs_setattr,
	.getattr	= ldiskfs_getattr,
	.listxattr	= ldiskfs_listxattr,
};

const struct inode_operations ldiskfs_fast_symlink_inode_operations = {
	.get_link	= simple_get_link,
	.setattr	= ldiskfs_setattr,
	.getattr	= ldiskfs_getattr,
	.listxattr	= ldiskfs_listxattr,
};
